#  ICE Revision: $Id: WriteDictionary.py 7855 2007-09-03 09:47:26Z bgschaid $ 
"""
Application class that implements pyFoamWriteDictionary
"""

import sys

from PyFoamApplication import PyFoamApplication

from PyFoam.RunDictionary.ParsedParameterFile import ParsedParameterFile

class WriteDictionary(PyFoamApplication):
    def __init__(self,args=None):
        description="""
Write a value to a Foam-Dictionary.
The description of the value is word. If the value is
non-atomic (a list or a dictionary) it has to be in in Python-notation.
Parts of the expression can be accessed with an option
        """
        
        PyFoamApplication.__init__(self,args=args,description=description,usage="%prog [options] <dictfile> <key> <val>",nr=3,interspersed=True)
        
    def addOptions(self):
        self.parser.add_option("--subexpression",action="store",default=None,dest="subexpression",help="A subexpression (in python notation to access parts of the value)")
        self.parser.add_option("--test",action="store_true",dest="test",default=False,help="Doesn't write to the file, but outputs the result on stdout")
        self.parser.add_option("--verbatim",action="store_true",dest="verbatim",default=False,help="Use the string without interpreting it as a Python-expression")
        
    
    def run(self):
        fName=self.parser.getArgs()[0]
        key=self.parser.getArgs()[1]
        val=self.parser.getArgs()[2]

        if self.opts.verbatim:
            newValue=val
        else:
            newValue=eval(val)
        
        try:
            dictFile=ParsedParameterFile(fName,backup=True)
            val=dictFile[key]
        except KeyError:
            self.error("Key: ",key,"not existing in File",fName)
        except IOError,e:
            self.error("Problem with file",fName,":",e)

        if self.opts.subexpression==None:
            dictFile[key]=newValue
        else:
            try:
                exec "dictFile[key]"+self.opts.subexpression+"=newValue"
            except Exception,e:
                self.error("Problem with subexpression:",sys.exc_info()[0],":",e)

        if self.opts.test:
            print str(dictFile)
        else:
            dictFile.writeFile()
        
